@@ -1,10 +1,9 @@
 import os
-from collections import OrderedDict
 from copy import deepcopy
 
-from ..utils.constants import DEFAULT_CONFIG_FILE
-from ..utils.fs import locate_file
-from .utils import get_normalized_dependency, is_valid_project_name, normalize_project_name
+from hatchling.metadata.utils import get_normalized_dependency, is_valid_project_name, normalize_project_name
+from hatchling.utils.constants import DEFAULT_CONFIG_FILE
+from hatchling.utils.fs import locate_file
 
 try:
     import tomllib
@@ -34,10 +33,14 @@ class ProjectMetadata:
         if config is not None and root is not None:
             self._project_file = os.path.join(root, 'pyproject.toml')
 
+    def has_project_file(self):
+        _ = self.config
+        return os.path.isfile(self._project_file)
+
     @property
     def context(self):
         if self._context is None:
-            from ..utils.context import Context
+            from hatchling.utils.context import Context
 
             self._context = Context(self.root)
 
@@ -86,18 +89,19 @@ class ProjectMetadata:
             if not isinstance(core_metadata, dict):
                 raise TypeError('The `project` configuration must be a table')
 
-            metadata = CoreMetadata(self.root, core_metadata, self.hatch.metadata)
+            metadata = CoreMetadata(self.root, core_metadata, self.hatch.metadata, self.context)
 
             metadata_hooks = self.hatch.metadata.hooks
             if metadata_hooks:
                 static_fields = set(core_metadata)
-                self._set_version(metadata)
-                core_metadata['version'] = self.version
+                if 'version' in self.hatch.config:
+                    self._set_version(metadata)
+                    core_metadata['version'] = self.version
 
                 for metadata_hook in metadata_hooks.values():
                     metadata_hook.update(core_metadata)
 
-                new_fields = set(core_metadata) - set(static_fields)
+                new_fields = set(core_metadata) - static_fields
                 for new_field in new_fields:
                     if new_field in metadata.dynamic:
                         metadata.dynamic.remove(new_field)
@@ -241,10 +245,11 @@ class CoreMetadata:
     https://peps.python.org/pep-0621/
     """
 
-    def __init__(self, root, config, hatch_metadata):
+    def __init__(self, root, config, hatch_metadata, context):
         self.root = root
         self.config = config
         self.hatch_metadata = hatch_metadata
+        self.context = context
 
         self._name = None
         self._version = None
@@ -269,6 +274,7 @@ class CoreMetadata:
         self._entry_points = None
         self._dependencies_complex = None
         self._dependencies = None
+        self._optional_dependencies_complex = None
         self._optional_dependencies = None
         self._dynamic = None
 
@@ -516,7 +522,7 @@ class CoreMetadata:
                 self._license = ''
                 self._license_expression = ''
             elif isinstance(data, str):
-                from ..licenses.parse import normalize_license_expression
+                from hatchling.licenses.parse import normalize_license_expression
 
                 self._license_expression = normalize_license_expression(data)
                 self._license = ''
@@ -825,7 +831,7 @@ class CoreMetadata:
             if not isinstance(urls, dict):
                 raise TypeError('Field `project.urls` must be a table')
 
-            sorted_urls = OrderedDict()
+            sorted_urls = {}
 
             for label, url in sorted(urls.items()):
                 if not isinstance(url, str):
@@ -856,7 +862,7 @@ class CoreMetadata:
             if not isinstance(scripts, dict):
                 raise TypeError('Field `project.scripts` must be a table')
 
-            sorted_scripts = OrderedDict()
+            sorted_scripts = {}
 
             for name, object_ref in sorted(scripts.items()):
                 if not isinstance(object_ref, str):
@@ -887,7 +893,7 @@ class CoreMetadata:
             if not isinstance(gui_scripts, dict):
                 raise TypeError('Field `project.gui-scripts` must be a table')
 
-            sorted_gui_scripts = OrderedDict()
+            sorted_gui_scripts = {}
 
             for name, object_ref in sorted(gui_scripts.items()):
                 if not isinstance(object_ref, str):
@@ -925,13 +931,13 @@ class CoreMetadata:
                         f'instead of in the `project.entry-points` table'
                     )
 
-            entry_point_groups = OrderedDict()
+            entry_point_groups = {}
 
             for group, entry_point_data in sorted(defined_entry_point_groups.items()):
                 if not isinstance(entry_point_data, dict):
                     raise TypeError(f'Field `project.entry-points.{group}` must be a table')
 
-                entry_points = OrderedDict()
+                entry_points = {}
 
                 for name, object_ref in sorted(entry_point_data.items()):
                     if not isinstance(object_ref, str):
@@ -969,14 +975,14 @@ class CoreMetadata:
             if not isinstance(dependencies, list):
                 raise TypeError('Field `project.dependencies` must be an array')
 
-            dependencies_complex = OrderedDict()
+            dependencies_complex = {}
 
             for i, entry in enumerate(dependencies, 1):
                 if not isinstance(entry, str):
                     raise TypeError(f'Dependency #{i} of field `project.dependencies` must be a string')
 
                 try:
-                    requirement = Requirement(entry)
+                    requirement = Requirement(self.context.format(entry))
                 except InvalidRequirement as e:
                     raise ValueError(f'Dependency #{i} of field `project.dependencies` is invalid: {e}')
                 else:
@@ -987,7 +993,7 @@ class CoreMetadata:
 
                     dependencies_complex[get_normalized_dependency(requirement)] = requirement
 
-            self._dependencies_complex = dependencies_complex
+            self._dependencies_complex = dict(sorted(dependencies_complex.items()))
 
         return self._dependencies_complex
 
@@ -997,16 +1003,16 @@ class CoreMetadata:
         https://peps.python.org/pep-0621/#dependencies-optional-dependencies
         """
         if self._dependencies is None:
-            self._dependencies = sorted(self.dependencies_complex)
+            self._dependencies = list(self.dependencies_complex)
 
         return self._dependencies
 
     @property
-    def optional_dependencies(self):
+    def optional_dependencies_complex(self):
         """
         https://peps.python.org/pep-0621/#dependencies-optional-dependencies
         """
-        if self._optional_dependencies is None:
+        if self._optional_dependencies_complex is None:
             from packaging.requirements import InvalidRequirement, Requirement
 
             if 'optional-dependencies' in self.config:
@@ -1037,7 +1043,7 @@ class CoreMetadata:
                         f'Dependencies for option `{option}` of field `project.optional-dependencies` must be an array'
                     )
 
-                entries = set()
+                entries = {}
 
                 for i, entry in enumerate(dependencies, 1):
                     if not isinstance(entry, str):
@@ -1047,7 +1053,7 @@ class CoreMetadata:
                         )
 
                     try:
-                        requirement = Requirement(entry)
+                        requirement = Requirement(self.context.format(entry))
                     except InvalidRequirement as e:
                         raise ValueError(
                             f'Dependency #{i} of option `{option}` of field `project.optional-dependencies` '
@@ -1060,7 +1066,7 @@ class CoreMetadata:
                                 f'cannot be a direct reference'
                             )
 
-                        entries.add(get_normalized_dependency(requirement))
+                        entries[get_normalized_dependency(requirement)] = requirement
 
                 normalized_option = normalize_project_name(option)
                 if normalized_option in normalized_options:
@@ -1070,9 +1076,21 @@ class CoreMetadata:
                     )
 
                 normalized_options[normalized_option] = option
-                optional_dependency_entries[normalized_option] = sorted(entries)
+                optional_dependency_entries[normalized_option] = dict(sorted(entries.items()))
 
-            self._optional_dependencies = OrderedDict(sorted(optional_dependency_entries.items()))
+            self._optional_dependencies_complex = dict(sorted(optional_dependency_entries.items()))
+
+        return self._optional_dependencies_complex
+
+    @property
+    def optional_dependencies(self):
+        """
+        https://peps.python.org/pep-0621/#dependencies-optional-dependencies
+        """
+        if self._optional_dependencies is None:
+            self._optional_dependencies = {
+                option: list(entries) for option, entries in self.optional_dependencies_complex.items()
+            }
 
         return self._optional_dependencies
 
@@ -1219,7 +1237,7 @@ class HatchVersionConfig:
             source_name = self.source_name
             version_source = self.plugin_manager.version_source.get(source_name)
             if version_source is None:
-                from ..plugin.exceptions import UnknownPluginError
+                from hatchling.plugin.exceptions import UnknownPluginError
 
                 raise UnknownPluginError(f'Unknown version source: {source_name}')
 
@@ -1235,7 +1253,7 @@ class HatchVersionConfig:
             scheme_name = self.scheme_name
             version_scheme = self.plugin_manager.version_scheme.get(scheme_name)
             if version_scheme is None:
-                from ..plugin.exceptions import UnknownPluginError
+                from hatchling.plugin.exceptions import UnknownPluginError
 
                 raise UnknownPluginError(f'Unknown version scheme: {scheme_name}')
 
@@ -1281,11 +1299,11 @@ class HatchMetadataSettings:
         if self._hooks is None:
             hook_config = self.hook_config
 
-            configured_hooks = OrderedDict()
+            configured_hooks = {}
             for hook_name, config in hook_config.items():
                 metadata_hook = self.plugin_manager.metadata_hook.get(hook_name)
                 if metadata_hook is None:
-                    from ..plugin.exceptions import UnknownPluginError
+                    from hatchling.plugin.exceptions import UnknownPluginError
 
                     raise UnknownPluginError(f'Unknown metadata hook: {hook_name}')
 
