@@ -1,7 +1,6 @@
 import os
-from io import open
 
-from .plugin.interface import VersionSourceInterface
+from hatchling.version.source.plugin.interface import VersionSourceInterface
 
 
 class CodeSource(VersionSourceInterface):
@@ -16,7 +15,7 @@ class CodeSource(VersionSourceInterface):
 
         path = os.path.normpath(os.path.join(self.root, relative_path))
         if not os.path.isfile(path):
-            raise OSError('file does not exist: {}'.format(relative_path))
+            raise OSError(f'file does not exist: {relative_path}')
 
         expression = self.config.get('expression') or '__version__'
         if not isinstance(expression, str):
@@ -25,7 +24,9 @@ class CodeSource(VersionSourceInterface):
         with open(path, 'r', encoding='utf-8') as f:
             contents = f.read()
 
-        global_variables = {}
+        # Ensure predefined module attributes are available.
+        # https://docs.python.org/3/reference/datamodel.html#the-standard-type-hierarchy.
+        global_variables = {'__file__': path, '__annotations__': dict()}
 
         # Load the file
         exec(contents, global_variables)
